@@ -6,8 +6,9 @@ use pyo3::pyclass::CompareOp;
 use pyo3::types::PyTuple;
 use pyo3::types::{PyDate, PyDateTime, PyDelta, PyDeltaAccess, PyDict, PyTime, PyTzInfo};
 use pyo3::IntoPyObjectExt;
-use speedate::MicrosecondsPrecisionOverflowBehavior;
-use speedate::{Date, DateTime, Duration, ParseError, Time, TimeConfig};
+use speedate::{
+    Date, DateTime, DateTimeConfig, Duration, MicrosecondsPrecisionOverflowBehavior, ParseError, Time, TimeConfig,
+};
 use std::borrow::Cow;
 use std::collections::hash_map::DefaultHasher;
 use std::fmt::Write;
@@ -366,9 +367,12 @@ pub fn bytes_as_datetime<'py>(
 ) -> ValResult<EitherDateTime<'py>> {
     match DateTime::parse_bytes_with_config(
         bytes,
-        &TimeConfig {
-            microseconds_precision_overflow_behavior: microseconds_overflow_behavior,
-            unix_timestamp_offset: Some(0),
+        &DateTimeConfig {
+            time_config: TimeConfig {
+                microseconds_precision_overflow_behavior: microseconds_overflow_behavior,
+                unix_timestamp_offset: Some(0),
+            },
+            ..Default::default()
         },
     ) {
         Ok(dt) => Ok(dt.into()),
@@ -390,8 +394,11 @@ pub fn int_as_datetime<'py>(
     match DateTime::from_timestamp_with_config(
         timestamp,
         timestamp_microseconds,
-        &TimeConfig {
-            unix_timestamp_offset: Some(0),
+        &DateTimeConfig {
+            time_config: TimeConfig {
+                unix_timestamp_offset: Some(0),
+                ..Default::default()
+            },
             ..Default::default()
         },
     ) {
