@@ -1,10 +1,11 @@
 import json
+from dataclasses import dataclass
 from datetime import date, datetime
 from uuid import UUID
 
 import pytest
 
-from pydantic_core import SchemaSerializer, SchemaValidator, core_schema
+from pydantic_core import SchemaSerializer, SchemaValidator, core_schema, to_json
 
 
 class TestBenchmarkSimpleModel:
@@ -394,3 +395,63 @@ def test_filter(benchmark):
     @benchmark
     def t():
         v.to_python(['a', 'b', 'c', 'd', 'e'], include={-1, -2})
+
+
+@pytest.mark.benchmark(group='list-of-lists')
+def test_to_json_list_of_lists(benchmark):
+    data = [[i + j for j in range(10)] for i in range(1000)]
+
+    benchmark(to_json, data)
+
+
+@pytest.mark.benchmark(group='list-of-lists')
+def test_ser_list_of_lists(benchmark):
+    s = SchemaSerializer(core_schema.list_schema(core_schema.list_schema(core_schema.int_schema())))
+    data = [[i + j for j in range(10)] for i in range(1000)]
+
+    benchmark(s.to_json, data)
+
+
+@dataclass
+class Foo:
+    a: str
+    b: bytes
+    c: int
+    d: float
+
+
+dataclass_schema = core_schema.dataclass_schema(
+    Foo,
+    core_schema.dataclass_args_schema(
+        'Foo',
+        [
+            core_schema.dataclass_field(name='a', schema=core_schema.str_schema()),
+            core_schema.dataclass_field(name='b', schema=core_schema.bytes_schema()),
+            core_schema.dataclass_field(name='c', schema=core_schema.int_schema()),
+            core_schema.dataclass_field(name='d', schema=core_schema.float_schema()),
+        ],
+    ),
+    ['a', 'b', 'c', 'd'],
+)
+
+
+@pytest.mark.benchmark(group='dataclass-ser')
+def test_dataclass_serialization_python(benchmark):
+    s = SchemaSerializer(dataclass_schema)
+    dc = Foo(a='hello', b=b'more', c=123, d=1.23)
+    assert s.to_python(dc) == {'a': 'hello', 'b': b'more', 'c': 123, 'd': 1.23}
+    benchmark(s.to_python, dc)
+
+
+@pytest.mark.benchmark(group='dataclass-ser')
+def test_dataclass_serialization_json(benchmark):
+    s = SchemaSerializer(dataclass_schema)
+    dc = Foo(a='hello', b=b'more', c=123, d=1.23)
+    assert s.to_python(dc) == {'a': 'hello', 'b': b'more', 'c': 123, 'd': 1.23}
+    benchmark(s.to_json, dc)
+
+
+@pytest.mark.benchmark(group='dataclass-ser')
+def test_dataclass_to_json(benchmark):
+    dc = Foo(a='hello', b=b'more', c=123, d=1.23)
+    benchmark(to_json, dc)
