@@ -1,12 +1,14 @@
+import platform
 import re
 import sys
 from functools import wraps
 from inspect import Parameter, signature
-from typing import Any, get_type_hints
+from typing import Any, Union, get_type_hints
 
 import pytest
 
 from pydantic_core import ArgsKwargs, SchemaError, SchemaValidator, ValidationError, core_schema
+from pydantic_core import core_schema as cs
 
 from ..conftest import Err, PyAndJson, plain_repr
 
@@ -371,6 +373,83 @@ def test_positional_or_keyword(py_and_json: PyAndJson, input_value, expected):
         assert v.validate_test(input_value) == expected
 
 
+@pytest.mark.parametrize(
+    'input_value,expected,arguments_schema',
+    [
+        (
+            {'a': 1, 'b': 2, 'e': 3.14},
+            ((), {'a': 1, 'b': 2, 'c': 5, 'd': 'default', 'e': 3.14}),
+            [
+                {'name': 'a', 'mode': 'positional_or_keyword', 'schema': {'type': 'int'}},
+                {'name': 'b', 'mode': 'positional_or_keyword', 'schema': {'type': 'int'}},
+                {
+                    'name': 'c',
+                    'mode': 'keyword_only',
+                    'schema': {'type': 'default', 'schema': {'type': 'int'}, 'default': 5},
+                },
+                {
+                    'name': 'd',
+                    'mode': 'keyword_only',
+                    'schema': {'type': 'default', 'schema': {'type': 'str'}, 'default': 'default'},
+                },
+                {'name': 'e', 'mode': 'keyword_only', 'schema': {'type': 'float'}},
+            ],
+        ),
+        (
+            {'y': 'test'},
+            ((), {'x': 1, 'y': 'test'}),
+            [
+                {
+                    'name': 'x',
+                    'mode': 'keyword_only',
+                    'schema': {'type': 'default', 'schema': {'type': 'int'}, 'default': 1},
+                },
+                {'name': 'y', 'mode': 'keyword_only', 'schema': {'type': 'str'}},
+            ],
+        ),
+        (
+            {'a': 1, 'd': 3.14},
+            ((), {'a': 1, 'b': 10, 'c': 'hello', 'd': 3.14}),
+            [
+                {'name': 'a', 'mode': 'positional_or_keyword', 'schema': {'type': 'int'}},
+                {
+                    'name': 'b',
+                    'mode': 'positional_or_keyword',
+                    'schema': {'type': 'default', 'schema': {'type': 'int'}, 'default': 10},
+                },
+                {
+                    'name': 'c',
+                    'mode': 'keyword_only',
+                    'schema': {'type': 'default', 'schema': {'type': 'str'}, 'default': 'hello'},
+                },
+                {'name': 'd', 'mode': 'keyword_only', 'schema': {'type': 'float'}},
+            ],
+        ),
+        (
+            {'x': 3, 'y': 'custom', 'z': 4},
+            ((), {'x': 3, 'y': 'custom', 'z': 4}),
+            [
+                {'name': 'x', 'mode': 'positional_or_keyword', 'schema': {'type': 'int'}},
+                {
+                    'name': 'y',
+                    'mode': 'keyword_only',
+                    'schema': {'type': 'default', 'schema': {'type': 'str'}, 'default': 'default'},
+                },
+                {'name': 'z', 'mode': 'keyword_only', 'schema': {'type': 'int'}},
+            ],
+        ),
+    ],
+)
+def test_keyword_only_non_default(py_and_json: PyAndJson, input_value, expected, arguments_schema):
+    v = py_and_json(
+        {
+            'type': 'arguments',
+            'arguments_schema': arguments_schema,
+        }
+    )
+    assert v.validate_test(input_value) == expected
+
+
 @pytest.mark.parametrize('input_value,expected', [[(1,), ((1,), {})], [(), ((42,), {})]], ids=repr)
 def test_positional_optional(py_and_json: PyAndJson, input_value, expected):
     v = py_and_json(
@@ -660,17 +739,16 @@ def test_default_factory(py_and_json: PyAndJson, input_value, expected):
 
 def test_repr():
     v = SchemaValidator(
-        {
-            'type': 'arguments',
-            'arguments_schema': [
-                {'name': 'b', 'mode': 'positional_or_keyword', 'schema': {'type': 'int'}},
+        cs.arguments_schema(
+            arguments=[
+                {'name': 'b', 'mode': 'positional_or_keyword', 'schema': cs.int_schema()},
                 {
                     'name': 'a',
                     'mode': 'keyword_only',
-                    'schema': {'type': 'default', 'schema': {'type': 'int'}, 'default_factory': lambda: 42},
+                    'schema': cs.with_default_schema(schema=cs.int_schema(), default_factory=lambda: 42),
                 },
-            ],
-        }
+            ]
+        )
     )
     assert 'positional_params_count:1,' in plain_repr(v)
 
@@ -678,20 +756,26 @@ def test_repr():
 def test_build_non_default_follows():
     with pytest.raises(SchemaError, match="Non-default argument 'b' follows default argument"):
         SchemaValidator(
-            {
-                'type': 'arguments',
-                'arguments_schema': [
+            schema=cs.arguments_schema(
+                arguments=[
                     {
                         'name': 'a',
                         'mode': 'positional_or_keyword',
-                        'schema': {'type': 'default', 'schema': {'type': 'int'}, 'default_factory': lambda: 42},
+                        'schema': cs.with_default_schema(schema=cs.int_schema(), default_factory=lambda: 42),
                     },
-                    {'name': 'b', 'mode': 'positional_or_keyword', 'schema': {'type': 'int'}},
-                ],
-            }
+                    {'name': 'b', 'mode': 'positional_or_keyword', 'schema': cs.int_schema()},
+                ]
+            )
         )
 
 
+def test_build_missing_var_kwargs():
+    with pytest.raises(
+        SchemaError, match="`var_kwargs_schema` must be specified when `var_kwargs_mode` is `'unpacked-typed-dict'`"
+    ):
+        SchemaValidator(cs.arguments_schema(arguments=[], var_kwargs_mode='unpacked-typed-dict'))
+
+
 @pytest.mark.parametrize(
     'input_value,expected',
     [
@@ -701,7 +785,7 @@ def test_build_non_default_follows():
     ],
     ids=repr,
 )
-def test_kwargs(py_and_json: PyAndJson, input_value, expected):
+def test_kwargs_uniform(py_and_json: PyAndJson, input_value, expected):
     v = py_and_json(
         {
             'type': 'arguments',
@@ -719,6 +803,48 @@ def test_kwargs(py_and_json: PyAndJson, input_value, expected):
         assert v.validate_test(input_value) == expected
 
 
+@pytest.mark.parametrize(
+    'input_value,expected',
+    [
+        [ArgsKwargs((), {'x': 1}), ((), {'x': 1})],
+        [ArgsKwargs((), {'x': 1.0}), Err('x\n  Input should be a valid integer [type=int_type,')],
+        [ArgsKwargs((), {'x': 1, 'z': 'str'}), ((), {'x': 1, 'y': 'str'})],
+        [ArgsKwargs((), {'x': 1, 'y': 'str'}), Err('y\n  Extra inputs are not permitted [type=extra_forbidden,')],
+    ],
+)
+def test_kwargs_typed_dict(py_and_json: PyAndJson, input_value, expected):
+    v = py_and_json(
+        {
+            'type': 'arguments',
+            'arguments_schema': [],
+            'var_kwargs_mode': 'unpacked-typed-dict',
+            'var_kwargs_schema': {
+                'type': 'typed-dict',
+                'fields': {
+                    'x': {
+                        'type': 'typed-dict-field',
+                        'schema': {'type': 'int', 'strict': True},
+                        'required': True,
+                    },
+                    'y': {
+                        'type': 'typed-dict-field',
+                        'schema': {'type': 'str'},
+                        'required': False,
+                        'validation_alias': 'z',
+                    },
+                },
+                'config': {'extra_fields_behavior': 'forbid'},
+            },
+        }
+    )
+
+    if isinstance(expected, Err):
+        with pytest.raises(ValidationError, match=re.escape(expected.message)):
+            v.validate_test(input_value)
+    else:
+        assert v.validate_test(input_value) == expected
+
+
 @pytest.mark.parametrize(
     'input_value,expected',
     [
@@ -758,14 +884,14 @@ def test_alias(py_and_json: PyAndJson, input_value, expected):
     ],
     ids=repr,
 )
-def test_alias_populate_by_name(py_and_json: PyAndJson, input_value, expected):
+def test_alias_validate_by_name(py_and_json: PyAndJson, input_value, expected):
     v = py_and_json(
         {
             'type': 'arguments',
             'arguments_schema': [
                 {'name': 'a', 'mode': 'positional_or_keyword', 'schema': {'type': 'int'}, 'alias': 'Foo'}
             ],
-            'populate_by_name': True,
+            'validate_by_name': True,
         }
     )
     if isinstance(expected, Err):
@@ -775,57 +901,85 @@ def test_alias_populate_by_name(py_and_json: PyAndJson, input_value, expected):
         assert v.validate_test(input_value) == expected
 
 
-def validate(function):
-    """
-    a demo validation decorator to test arguments
-    """
-    parameters = signature(function).parameters
+def test_only_validate_by_name(py_and_json) -> None:
+    schema = core_schema.arguments_schema(
+        [
+            core_schema.arguments_parameter(name='a', schema=core_schema.str_schema(), alias='FieldA'),
+        ],
+        validate_by_name=True,
+        validate_by_alias=False,
+    )
+    v = py_and_json(schema)
+    assert v.validate_test(ArgsKwargs((), {'a': 'hello'})) == ((), {'a': 'hello'})
+    with pytest.raises(ValidationError, match=r'a\n +Missing required argument \[type=missing_argument,'):
+        assert v.validate_test(ArgsKwargs((), {'FieldA': 'hello'}))
 
-    type_hints = get_type_hints(function)
-    mode_lookup = {
-        Parameter.POSITIONAL_ONLY: 'positional_only',
-        Parameter.POSITIONAL_OR_KEYWORD: 'positional_or_keyword',
-        Parameter.KEYWORD_ONLY: 'keyword_only',
-    }
 
-    arguments_schema = []
-    schema = {'type': 'arguments', 'arguments_schema': arguments_schema}
-    for i, (name, p) in enumerate(parameters.items()):
-        if p.annotation is p.empty:
-            annotation = Any
-        else:
-            annotation = type_hints[name]
+def test_only_allow_alias(py_and_json) -> None:
+    schema = core_schema.arguments_schema(
+        [
+            core_schema.arguments_parameter(name='a', schema=core_schema.str_schema(), alias='FieldA'),
+        ],
+        validate_by_name=False,
+        validate_by_alias=True,
+    )
+    v = py_and_json(schema)
+    assert v.validate_test(ArgsKwargs((), {'FieldA': 'hello'})) == ((), {'a': 'hello'})
+    with pytest.raises(ValidationError, match=r'FieldA\n +Missing required argument \[type=missing_argument,'):
+        assert v.validate_test(ArgsKwargs((), {'a': 'hello'}))
 
-        assert annotation in (bool, int, float, str, Any), f'schema for {annotation} not implemented'
-        if annotation in (bool, int, float, str):
-            arg_schema = {'type': annotation.__name__}
-        else:
-            assert annotation is Any
-            arg_schema = {'type': 'any'}
 
-        if p.kind in mode_lookup:
-            if p.default is not p.empty:
-                arg_schema = {'type': 'default', 'schema': arg_schema, 'default': p.default}
-            s = {'name': name, 'mode': mode_lookup[p.kind], 'schema': arg_schema}
-            arguments_schema.append(s)
-        elif p.kind == Parameter.VAR_POSITIONAL:
-            schema['var_args_schema'] = arg_schema
-        else:
-            assert p.kind == Parameter.VAR_KEYWORD, p.kind
-            schema['var_kwargs_schema'] = arg_schema
+def validate(config=None):
+    def decorator(function):
+        parameters = signature(function).parameters
+        type_hints = get_type_hints(function)
+        mode_lookup = {
+            Parameter.POSITIONAL_ONLY: 'positional_only',
+            Parameter.POSITIONAL_OR_KEYWORD: 'positional_or_keyword',
+            Parameter.KEYWORD_ONLY: 'keyword_only',
+        }
 
-    validator = SchemaValidator(schema)
+        arguments_schema = []
+        schema = {'type': 'arguments', 'arguments_schema': arguments_schema}
+        for i, (name, p) in enumerate(parameters.items()):
+            if p.annotation is p.empty:
+                annotation = Any
+            else:
+                annotation = type_hints[name]
 
-    @wraps(function)
-    def wrapper(*args, **kwargs):
-        validated_args, validated_kwargs = validator.validate_python(ArgsKwargs(args, kwargs))
-        return function(*validated_args, **validated_kwargs)
+            assert annotation in (bool, int, float, str, Any), f'schema for {annotation} not implemented'
+            if annotation in (bool, int, float, str):
+                arg_schema = {'type': annotation.__name__}
+            else:
+                assert annotation is Any
+                arg_schema = {'type': 'any'}
 
-    return wrapper
+            if p.kind in mode_lookup:
+                if p.default is not p.empty:
+                    arg_schema = {'type': 'default', 'schema': arg_schema, 'default': p.default}
+                s = {'name': name, 'mode': mode_lookup[p.kind], 'schema': arg_schema}
+                arguments_schema.append(s)
+            elif p.kind == Parameter.VAR_POSITIONAL:
+                schema['var_args_schema'] = arg_schema
+            else:
+                assert p.kind == Parameter.VAR_KEYWORD, p.kind
+                schema['var_kwargs_schema'] = arg_schema
+
+        validator = SchemaValidator(schema, config=config)
+
+        @wraps(function)
+        def wrapper(*args, **kwargs):
+            # Validate arguments using the original schema
+            validated_args, validated_kwargs = validator.validate_python(ArgsKwargs(args, kwargs))
+            return function(*validated_args, **validated_kwargs)
+
+        return wrapper
+
+    return decorator
 
 
 def test_function_any():
-    @validate
+    @validate()
     def foobar(a, b, c):
         return a, b, c
 
@@ -842,7 +996,7 @@ def test_function_any():
 
 
 def test_function_types():
-    @validate
+    @validate()
     def foobar(a: int, b: int, *, c: int):
         return a, b, c
 
@@ -894,8 +1048,8 @@ def test_function_positional_only(import_execute):
     # language=Python
     m = import_execute(
         """
-def create_function(validate):
-    @validate
+def create_function(validate, config = None):
+    @validate(config = config)
     def foobar(a: int, b: int, /, c: int):
         return a, b, c
     return foobar
@@ -915,6 +1069,12 @@ def create_function(validate):
         },
         {'type': 'unexpected_keyword_argument', 'loc': ('b',), 'msg': 'Unexpected keyword argument', 'input': 2},
     ]
+    # Allowing extras using the config
+    foobar = m.create_function(validate, config={'title': 'func', 'extra_fields_behavior': 'allow'})
+    assert foobar('1', '2', c=3, d=4) == (1, 2, 3)
+    # Ignore works similar than allow
+    foobar = m.create_function(validate, config={'title': 'func', 'extra_fields_behavior': 'ignore'})
+    assert foobar('1', '2', c=3, d=4) == (1, 2, 3)
 
 
 @pytest.mark.skipif(sys.version_info < (3, 10), reason='requires python3.10 or higher')
@@ -923,7 +1083,7 @@ def test_function_positional_only_default(import_execute):
     m = import_execute(
         """
 def create_function(validate):
-    @validate
+    @validate()
     def foobar(a: int, b: int = 42, /):
         return a, b
     return foobar
@@ -940,7 +1100,7 @@ def test_function_positional_kwargs(import_execute):
     m = import_execute(
         """
 def create_function(validate):
-    @validate
+    @validate()
     def foobar(a: int, b: int, /, **kwargs: bool):
         return a, b, kwargs
     return foobar
@@ -953,7 +1113,7 @@ def create_function(validate):
 
 
 def test_function_args_kwargs():
-    @validate
+    @validate()
     def foobar(*args, **kwargs):
         return args, kwargs
 
@@ -966,24 +1126,21 @@ def test_function_args_kwargs():
 def test_invalid_schema():
     with pytest.raises(SchemaError, match="'default' and 'default_factory' cannot be used together"):
         SchemaValidator(
-            {
-                'type': 'arguments',
-                'arguments_schema': [
+            schema=cs.arguments_schema(
+                arguments=[
                     {
                         'name': 'a',
                         'mode': 'positional_or_keyword',
-                        'schema': {
-                            'type': 'default',
-                            'schema': {'type': 'int'},
-                            'default': 1,
-                            'default_factory': lambda: 2,
-                        },
+                        'schema': cs.with_default_schema(schema=cs.int_schema(), default=1, default_factory=lambda: 2),
                     }
-                ],
-            }
+                ]
+            )
         )
 
 
+@pytest.mark.xfail(
+    platform.python_implementation() == 'PyPy' and sys.version_info[:2] == (3, 11), reason='pypy 3.11 type formatting'
+)
 def test_error_display(pydantic_version):
     v = SchemaValidator(
         core_schema.arguments_schema(
@@ -1020,3 +1177,50 @@ def test_error_display(pydantic_version):
         '[{"type":"missing_argument","loc":["b"],"msg":"Missing required argument",'
         '"input":"ArgsKwargs((), {\'a\': 1})"}]'
     )
+
+
+@pytest.mark.parametrize('config_by_alias', [None, True, False])
+@pytest.mark.parametrize('config_by_name', [None, True, False])
+@pytest.mark.parametrize('runtime_by_alias', [None, True, False])
+@pytest.mark.parametrize('runtime_by_name', [None, True, False])
+def test_by_alias_and_name_config_interaction(
+    config_by_alias: Union[bool, None],
+    config_by_name: Union[bool, None],
+    runtime_by_alias: Union[bool, None],
+    runtime_by_name: Union[bool, None],
+) -> None:
+    """This test reflects the priority that applies for config vs runtime validation alias configuration.
+
+    Runtime values take precedence over config values, when set.
+    By default, by_alias is True and by_name is False.
+    """
+
+    if config_by_alias is False and config_by_name is False and runtime_by_alias is False and runtime_by_name is False:
+        pytest.skip("Can't have both by_alias and by_name as effectively False")
+
+    schema = core_schema.arguments_schema(
+        arguments=[
+            core_schema.arguments_parameter(name='my_field', schema=core_schema.int_schema(), alias='my_alias'),
+        ],
+        **({'validate_by_alias': config_by_alias} if config_by_alias is not None else {}),
+        **({'validate_by_name': config_by_name} if config_by_name is not None else {}),
+    )
+    s = SchemaValidator(schema)
+
+    alias_allowed = next(x for x in (runtime_by_alias, config_by_alias, True) if x is not None)
+    name_allowed = next(x for x in (runtime_by_name, config_by_name, False) if x is not None)
+
+    if alias_allowed:
+        assert s.validate_python(
+            ArgsKwargs((), {'my_alias': 1}), by_alias=runtime_by_alias, by_name=runtime_by_name
+        ) == (
+            (),
+            {'my_field': 1},
+        )
+    if name_allowed:
+        assert s.validate_python(
+            ArgsKwargs((), {'my_field': 1}), by_alias=runtime_by_alias, by_name=runtime_by_name
+        ) == (
+            (),
+            {'my_field': 1},
+        )
